using System;
using Unity.Netcode;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.InputSystem;

public class Character : NetworkBehaviour
{
    /// <summary>
    /// Kinematic character controller to move the player character
    /// as a kinematic object
    /// </summary>

    [Header("Input Controls")]

    /// <summary>
    /// Action reference for moving the player.
    /// </summary>
    [Tooltip("Action reference for moving the player")]
    [SerializeField]
    private InputActionReference moveAction;

    /// <summary>
    /// Action reference for jumping.
    /// </summary>
    [Tooltip("Action reference for moving the player")]
    [SerializeField]
    private InputActionReference jumpAction;

    /// <summary>
    /// Action reference for sprinting.
    /// </summary>
    [Tooltip("Action reference for moving the player")]
    [SerializeField]
    private InputActionReference sprintAction;



    [Header("Ground Checking")]

    /// <summary>
    /// Distance to ground at which player is considered grounded
    /// </summary>
    [Tooltip("Distance from ground at which a player is considered standing on the ground")]
    [SerializeField]
    private float groundedDistance = 0.01f;

    /// <summary>
    /// Distance to ground at which player is considered standing on something
    /// </summary>
    [Tooltip("Distance to ground at which player is considered standing on something")]
    [SerializeField]
    private float standingDistance = 0.1f;

    /// <summary>
    /// Distance to check player distance to ground
    /// </summary>
    [Tooltip("Distance to draw rays down when checking if player is grounded")]
    [SerializeField]
    private float groundCheckDistance = 5f;

    /// <summary>
    /// Maximum angle at which the player can walk (in degrees)
    /// </summary>
    [Tooltip("Maximum angle at which the player can walk")]
    [SerializeField]
    [Range(0, 90)]
    private float maxWalkAngle = 60f;

    /// <summary>
    /// Direction and strength of gravity
    /// </summary>
    [Tooltip("Direction and strength of gravity in units per second squared")]
    [SerializeField]
    private Vector3 gravity = new Vector3(0, -9.807f, 0);


    [Header("Motion Settings")]

    /// <summary>
    /// Speed of player movement when walking.
    /// </summary>
    [Tooltip("Speed of player when walking")]
    [SerializeField]
    private float walkingSpeed = 7.5f;

    /// <summary>
    /// Speed of player when sprinting.
    /// </summary>
    [Tooltip("Speed of player when sprinting")]
    [SerializeField]
    private float sprintSpeed = 10.0f;

    [Header("Objects")]
    [SerializeField] protected CharacterController _characterController;
    [SerializeField] protected Camera _mainCam;
    [SerializeField] protected AudioListener _audioListener;
    [SerializeField] protected Weapon _weapon;

    [Header("Input Actions")]
    [SerializeField] protected PlayerInput _playerInput;

    private void Awake()
    {
        _characterController = GetComponent<CharacterController>();
        _playerInput = GetComponent<PlayerInput>();
        _audioListener = GetComponentInChildren<AudioListener>();
    }


    // Public Inputs Methods

    // public void OnMove(InputAction.CallbackContext context)
    // {
    //     _moveInput = context.ReadValue<Vector2>();
    // }

    // public void OnLook(InputAction.CallbackContext context)
    // {
    //     _lookInput = context.ReadValue<Vector2>();
    // }

    // public void OnSprint(InputAction.CallbackContext context)
    // {
    //     if (context.performed)
    //     {
    //         Debug.Log("Running");
    //         _speedMultiplier = _sprintMultiplier;
    //     }
    //     else
    //     {
    //         _speedMultiplier = 1f;
    //     }
    // }

    // public void OnJump(InputAction.CallbackContext context)
    // {
    //     if (context.performed)
    //     {
    //         JumpServerRPC();
    //     }
    // }

    // public void OnAttack(InputAction.CallbackContext context)
    // {
    //     if (context.performed)
    //     {
    //         ShootServerRPC();
    //     }
    // }

    // public void OnAim(InputAction.CallbackContext context)
    // {
    //     if (context.performed)
    //     {
    //         AimServerRPC();
    //     }
    // }

    // public void OnCrouch(InputAction.CallbackContext context)
    // {
    //     if (context.performed)
    //     {
    //         CrouchServerRPC();
    //     }
    // }

    // public void OnInteract(InputAction.CallbackContext context)
    // {
    //     if (context.performed)
    //     {
    //         InteractServerRPC();
    //     }
    // }

    

    // Networking Staff
    // [ServerRpc]
    // protected void JumpServerRPC()
    // {
    //     if (_characterController.isGrounded)
    //     {
    //         Debug.Log($"Jump triggered. Owner: {OwnerClientId}");
    //     }
    // }

    // [ServerRpc]
    // protected void ShootServerRPC()
    // {
    //     Debug.Log($"Shooting triggered. Owner: {OwnerClientId}");
    //     _weapon.Shooting();
    // }

    // [ServerRpc]
    // protected void AimServerRPC()
    // {
    //     Debug.Log($"Aim triggered. Owner: {OwnerClientId}");
    // }

    // [ServerRpc]
    // protected void CrouchServerRPC()
    // {
    //     Debug.Log($"Crouch triggered. Owner: {OwnerClientId}");
    // }

    // [ServerRpc]
    // protected void InteractServerRPC()
    // {
    //     Debug.Log($"Interact triggered. Owner: {OwnerClientId}");
    // }
}
